//===========================================================================
// GroundUnit.ino - implements the code for Sailplane ground unit.
//   Tone generation code based on source from Rolf R Bakke, Oct 2012
//   T.W. Studwell Feb. 21, 2015
//===========================================================================
#include <RFM69.h>
#include <SPI.h>
#include "Sailplane.h"

#define TONE_PIN 5

RFM69 
   radio;

float 
   toneFreq, 
   toneFreqLowpass, 
   pressure=0.0, 
   lowpassFast=0.0, 
   lowpassSlow=0.0;

int 
   ddsAcc;

unsigned long 
   now,
   timeout = 0;


//=====================================================================================
// setup() - called once on startup
//=====================================================================================
void setup() {
  pinMode(LED_PIN,OUTPUT);
  pinMode(TONE_PIN,OUTPUT);
  pressure=lowpassFast=lowpassSlow=100000.0;  // init the data (1000.00mBar)
  
  Serial.begin(SERIAL_BAUD);

  delay(10);

  radio.initialize(FREQUENCY,GROUNDUNIT_ID,NETWORKID);
  radio.setHighPower(); //uncomment only for RFM69HW!
  radio.encrypt(ENCRYPT_KEY);
  char buff[50];
  sprintf(buff, "\nListening at %d Mhz...", FREQUENCY==RF69_433MHZ ? 433 : FREQUENCY==RF69_868MHZ ? 868 : 915);
  Serial.println(buff);

}


//=====================================================================================
// loop() called constantly, immediately after returning because Arduino doesn't expect their users to know how to do this on their own ;-)
//=====================================================================================
void loop() {

  now = millis();
  
  //====================================================================
  // On this side of the 'network', we want to listen for pressure transmissions
  // and then produce a tone based on that pressure.   
  // We don't even need to keep track of timing
  // since it will be governed by the sailplane transmissions...
  //====================================================================
  if (radio.receiveDone())
  {
    // ok!  Got data!  It MUST be a pressure reading!  We can confirm by checking data length and source
    if (radio.DATALEN == sizeof(float) && radio.SENDERID == SAILPLANE_ID)
    {
      // we're good to go! (probably could be more thorough about this, like put a 'signature' in the packet or something)
      memcpy(&pressure,(const byte*)radio.DATA,radio.DATALEN);  // grab the data
      timeout = now + (3 * SAMPLE_INTERVAL);                    // reset timer
      
      if (radio.ACKRequested())  // this SHOULDN'T HAPPEN, but... in case it does, ack the packet
      {
        radio.sendACK();
        Serial.print(" - ACK sent.");
      }
      Blink(LED_PIN,5);
      
      // ok, now generate the tone
        
      lowpassFast = lowpassFast + (pressure - lowpassFast) * 0.1;
      lowpassSlow = lowpassSlow + (pressure - lowpassSlow) * 0.05;
      
      toneFreq = (lowpassSlow - lowpassFast) * 50;
      
      toneFreqLowpass = toneFreqLowpass + (toneFreq - toneFreqLowpass) * 0.1;
       
      toneFreq = constrain(toneFreqLowpass, -500, 500);
      
      ddsAcc += toneFreq * 100 + 2000;
      
      if (toneFreq < 0 || ddsAcc > 0) 
      {
        pinMode(TONE_PIN,OUTPUT);
        tone(TONE_PIN, toneFreq + 510);  
     }
      else
      {
    //    noTone(TONE_PIN);
          pinMode(TONE_PIN,INPUT);  // go to tristate instead of turning off tone to avoid clicks
      }
    }
  }
  if (now > timeout)
  {
    // we've gone three sample times without a packet from the sailplane - squelch the tone
    timeout = now + (3 * SAMPLE_INTERVAL);                    // reset timer

    pinMode(TONE_PIN,INPUT);  // go to tristate instead of turning off tone to avoid clicks
    noTone(TONE_PIN);
  }

}


//=====================================================================================
// Blink() - the blinkin led...
//=====================================================================================
void Blink(byte pin, int Delay_mS)
{
  digitalWrite(pin,HIGH);
  delay(Delay_mS);
  digitalWrite(pin,LOW);
}
