//===========================================================================
// Sailplane.ino - implements the code for Sailplane in-flight pressure/altimeter sensor.
//   Sensor acquisition and calibration code based on source from Rolf R Bakke, Oct 2012
//   T.W. Studwell Feb. 21, 2015
//===========================================================================
#include <RFM69.h>
#include <SPI.h>
#include <Wire.h>
#include "Sailplane.h"

// this code is based on the RFM69 library example called Struct_send()
// a lot was taken out to simplify, for example, SPIflash - didn't have an obvious use for it
// 

RFM69 radio;

unsigned int 
   calibrationData[7];
unsigned long 
   now,
   timeout = 0;

float pressure;

//=====================================================================================
// setup() - called once on startup
//=====================================================================================
void setup() {
  pinMode(LED_PIN,OUTPUT);
  
  Wire.begin();
  Serial.begin(SERIAL_BAUD);
  
  setupSensor();

  pressure = getPressure();

  radio.initialize(FREQUENCY,SAILPLANE_ID,NETWORKID);
  radio.setHighPower(); //uncomment only for RFM69HW!
  radio.encrypt(ENCRYPT_KEY);
  char buff[50];
  sprintf(buff, "\nTransmitting at %d Mhz...", FREQUENCY==RF69_433MHZ ? 433 : FREQUENCY==RF69_868MHZ ? 868 : 915);
  Serial.println(buff);
  
}


//=====================================================================================
// loop() called constantly, immediately after returning because Arduino doesn't expect their users to know how to do this on their own ;-)
//=====================================================================================
void loop() {
  
  //#######################################################################################
  //################### THIS CODE BLOCK CAN PROBABLY GO AWAY UNLESS...#####################
  //######## YOU WANT TO SEND COMMANDS TO CONTROL WHEN THE SAILPLANE TRANSMITS PRESSURE####
  //check for any received packets (shouldn't be any but we'll leave the code here for now)
  if (radio.receiveDone())
  {
    Serial.print('[');Serial.print(radio.SENDERID, DEC);Serial.print("] ");
    for (byte i = 0; i < radio.DATALEN; i++)
      Serial.print((char)radio.DATA[i]);
    Serial.print("   [RX_RSSI:");Serial.print(radio.readRSSI());Serial.print("]");

    if (radio.ACKRequested())
    {
      radio.sendACK();
      Serial.print(" - ACK sent");
      delay(10);
    }
    Serial.println();
  }
  //################### THIS CODE BLOCK CAN PROBABLY GO AWAY SEE ABOVE#####################
  //#######################################################################################
  
  //################### THIS CODE BLOCK STAYS #############################################
  // now we begin the REAL sampling loop, note that the tone generation has been moved to GroundUnit!
  now = millis();
  
  if (now > timeout)
  {
    pressure = getPressure();  // get the calibrated reading

    // we don't use send with retry because this data is 'isochronous' (continuous data)
    // therefore, if we lose a sample, we can't worry about it because there's another coming up soon
    radio.send(GROUNDUNIT_ID, (const void*)(&pressure), sizeof(pressure));  
    timeout = now + SAMPLE_INTERVAL;
  }
}


//=====================================================================================
// getPressure() - function to read the MS5611 and then calculate a calibrated pressure
//=====================================================================================
long getPressure()
{
  long D1, D2, dT, P;
  float fP;
  float TEMP;
  int64_t OFF, SENS;

  D1 = getData(0x48, 10);
  D2 = getData(0x50, 1);

  dT = D2 - ((long)calibrationData[5] << 8);
  TEMP = (2000 + (((int64_t)dT * (int64_t)calibrationData[6]) >> 23)) / (float)100;
  OFF = ((unsigned long)calibrationData[2] << 16) + (((int64_t)calibrationData[4] * dT) >> 7);
  SENS = ((unsigned long)calibrationData[1] << 15) + (((int64_t)calibrationData[3] * dT) >> 8);
  P = (((D1 * SENS) >> 21) - OFF) >> 15;
  
  return P;
}


//=====================================================================================
// getData() - function to read 24 bits from the sensor, which bits is controlled by command, del controls the wait time before reading.
//=====================================================================================
long getData(byte command, byte del)
{
  long result = 0;
  twiSendCommand(0x77, command);
  delay(del);
  twiSendCommand(0x77, 0x00);
  Wire.requestFrom(0x77, 3);
  if(Wire.available()!=3) Serial.println("Error: raw data not available");
  for (int i = 0; i <= 2; i++)
  {
    result = (result<<8) | Wire.read(); 
  }
  return result;
}



//=====================================================================================
// setupSensor() - called once on startup
//=====================================================================================
void setupSensor()
{
  twiSendCommand(0x77, 0x1e);
  delay(100);
  
  for (byte i = 1; i <=6; i++)
  {
    unsigned int low, high;

    twiSendCommand(0x77, 0xa0 + i * 2);
    Wire.requestFrom(0x77, 2);
    if(Wire.available()!=2) Serial.println("Error: calibration data not available");
    high = Wire.read();
    low = Wire.read();
    calibrationData[i] = high<<8 | low;
    Serial.print("calibration data #");
    Serial.print(i);
    Serial.print(" = ");
    Serial.println( calibrationData[i] ); 
  }
}



//=====================================================================================
// twiSendCommand() - uses two wire libary to talk to the MS5611 I2C interface
//=====================================================================================
void twiSendCommand(byte address, byte command)
{
  Wire.beginTransmission(address);
  if (!Wire.write(command)) Serial.println("Error: write()");
  if (Wire.endTransmission()) 
  {
    Serial.print("Error when sending command: ");
    Serial.println(command, HEX);
  }
}



//=====================================================================================
// ledOn() - guess
//=====================================================================================
void ledOn()
{
  digitalWrite(LED_PIN,1);
}



//=====================================================================================
// ledOff() - guess again
//=====================================================================================
void ledOff()
{
  digitalWrite(LED_PIN,0);
}
